from abc import ABC, abstractmethod


class S3Client(ABC):

    @abstractmethod
    def get_object(self, bucket: str, key: str) -> bytes:
        pass

    @abstractmethod
    def put_object(self, bucket: str, key: str, data: bytes):
        pass

    @abstractmethod
    def delete_object(self, bucket: str, key: str):
        pass

    @abstractmethod
    def list_objects(self, bucket: str) -> list:
        pass

    @abstractmethod
    def list_buckets(self) -> list:
        pass

    @abstractmethod
    def create_bucket(self, bucket: str):
        pass

    @abstractmethod
    def delete_bucket(self, bucket: str):
        pass

    @abstractmethod
    def get_bucket_location(self, bucket: str) -> str:
        pass

    @abstractmethod
    def get_signed_url(self, bucket: str, key: str, expiration: int) -> str:
        pass



from finalsa.s3.client.client import S3ClientImpl
from finalsa.s3.client.tests import S3ClientTest
from finalsa.s3.client.interface import S3Client

__version__ = "0.1.2"

__all__ = [
    "S3ClientImpl",
    "S3ClientTest",
    "S3Client"
]



from finalsa.s3.client.interface import S3Client


class S3ClientTest(S3Client):

    def __init__(self) -> None:
        self.buckets = {}

    def get_object(self, bucket: str, key: str) -> bytes:
        return self.buckets[bucket][key]

    def put_object(self, bucket: str, key: str, data: bytes):
        if bucket not in self.buckets:
            self.buckets[bucket] = {}
        self.buckets[bucket][key] = data

    def delete_object(self, bucket: str, key: str):
        del self.buckets[bucket][key]

    def list_objects(self, bucket: str) -> list:
        return self.buckets[bucket].keys()

    def list_buckets(self) -> list:
        return self.buckets.keys()

    def create_bucket(self, bucket: str):
        self.buckets[bucket] = {}

    def delete_bucket(self, bucket: str):
        del self.buckets[bucket]

    def get_bucket_location(self, bucket: str) -> str:
        return "us-east-1"

    def get_signed_url(self, bucket: str, key: str, expiration: int) -> str:
        return f"https://s3.amazonaws.com/{bucket}/{key}?Expires={expiration}"

    def clear(self):
        self.buckets = {}



from finalsa.s3.client import (
    S3Client,
    S3ClientImpl,
    S3ClientTest,
    __version__
)
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../../')))


def test_version():
    assert __version__ is not None


def test_client():
    assert S3Client is not None


def test_client_impl():
    assert S3ClientImpl is not None


def test_client_test():
    client = S3ClientTest()
    assert client is not None


def test_client_get_object():
    client = S3ClientTest()
    client.put_object("bucket", "key", b"data")
    assert client.get_object("bucket", "key") == b"data"


def test_client_put_object():
    client = S3ClientTest()
    client.put_object("bucket", "key", b"data")
    assert client.get_object("bucket", "key") == b"data"


def test_client_delete_object():
    client = S3ClientTest()
    client.put_object("bucket", "key", b"data")
    client.delete_object("bucket", "key")

    assert "key" not in client.list_objects("bucket")


def test_client_list_objects():
    client = S3ClientTest()
    client.put_object("bucket", "key", b"data")
    client.put_object("bucket", "key2", b"data")

    assert set(client.list_objects("bucket")) == {"key", "key2"}


def test_get_signed_url():
    client = S3ClientTest()
    client.put_object("bucket", "key", b"data")
    assert client.get_signed_url(
        "bucket", "key", 10) == "https://s3.amazonaws.com/bucket/key?Expires=10"



from finalsa.s3.client.interface.client import S3Client
import boto3
from botocore.config import Config
import logging


class S3ClientImpl(S3Client):

    def __init__(
        self,
    ) -> None:
        config = Config(signature_version='s3v4')
        self.client = boto3.client('s3', config=config)
        self.logger = logging.getLogger("finalsa.clients")

    def get_object(self, bucket: str, key: str) -> bytes:
        self.logger.info(f"Getting object from bucket {bucket} with key {key}")
        response = self.client.get_object(Bucket=bucket, Key=key)
        return response['Body'].read()

    def put_object(self, bucket: str, key: str, data: bytes):
        self.logger.info(f"Putting object to bucket {bucket} with key {key}")
        self.client.put_object(Bucket=bucket, Key=key, Body=data)

    def delete_object(self, bucket: str, key: str):
        self.logger.info(f"Deleting object from bucket {bucket} with key {key}")
        self.client.delete_object(Bucket=bucket, Key=key)

    def list_objects(self, bucket: str) -> list:
        self.logger.info(f"Listing objects from bucket {bucket}")
        response = self.client.list_objects(Bucket=bucket)
        return response['Contents']

    def list_buckets(self) -> list:
        self.logger.info(f"Listing buckets")
        response = self.client.list_buckets()
        return response['Buckets']

    def create_bucket(self, bucket: str):
        self.logger.info(f"Creating bucket {bucket}")
        self.client.create_bucket(Bucket=bucket)

    def delete_bucket(self, bucket: str):
        self.logger.info(f"Deleting bucket {bucket}")
        self.client.delete_bucket(Bucket=bucket)

    def get_bucket_location(self, bucket: str) -> str:
        self.logger.info(f"Getting location of bucket {bucket}")
        response = self.client.get_bucket_location(Bucket=bucket)
        return response['LocationConstraint']

    def get_signed_url(self, bucket: str, key: str, expiration: int) -> str:
        self.logger.info(f"Getting signed url for bucket {bucket} with key {key}")
        url = self.client.generate_presigned_url(
            ClientMethod='get_object',
            Params={'Bucket': bucket, 'Key': key},
            ExpiresIn=expiration)
        return url
